module RUser
  # This class allows the creation of new users from the RandomUser.me API.
  # @author Joshua Kendall
  class Person
    KEYS = {
      'zip' => 'postal',
      'first' => 'first_name',
      'last' => 'last_name',
      'large' => 'profile_large_url',
      'medium' => 'profile_medium_url',
      'thumbnail' => 'profile_thumbnail_url'
    }.freeze

    NIDT = %w(TFN DNI HETU INSEE PPS BSN SSN NINO).freeze

    # @return [String] the user's gender, either male or female.
    attr_reader :gender

    # @return [String] the user's title (Mr, Mrs, etc.).
    attr_reader :title

    # @return [String] the user's first name.
    attr_reader :first_name

    # @return [String] the user's last name.
    attr_reader :last_name

    # @return [String] the user's street address.
    attr_reader :street

    # @return [String] the user's city.
    attr_reader :city

    # @return [String] the user's state.
    attr_reader :state

    # @return [String] the user's zip code or postal code.
    attr_reader :postal

    # @return [String] the user's email.
    attr_reader :email

    # @return [String] the user's username.
    attr_reader :username

    # @return [String] the user's password.
    attr_reader :password

    # @return [String] the user's password salt.
    attr_reader :salt

    # @return [String] the user's md5 hash of the user's salted password.
    attr_reader :md5

    # @return [String] the user's sha1 hash of the user's salted password.
    attr_reader :sha1

    # @return [String] the user's sha256 hash of the user's salted password.
    attr_reader :sha256

    # @return [String] the user's registration date.
    attr_reader :registered

    # @return [String] the user's birth date.
    attr_reader :dob

    # @return [String] the user's phone number.
    attr_reader :phone

    # @return [String] the user's cell phone number.
    attr_reader :cell

    # @return [String] the user's national Id number (SSN, PPS, HETU,, etc.).
    attr_reader :nidn

    # @return [String] the user's national Id type (SSN, PPS, HETU, NINO, etc.).
    attr_reader :nidt

    # @return [String] the user's large profile photo url
    attr_reader :profile_large_url

    # @return [String] the user's medium profile photo url
    attr_reader :profile_medium_url

    # @return [String] the user's thumbnail profile photo url
    attr_reader :profile_thumbnail_url

    # @return [String] the user's nationality
    attr_reader :nationality

    # @return [String] the user's unique seed which allows future calls.
    attr_reader :seed

    # Creates a new person object
    #
    # @param [Hash] data the data used to create the user
    # @return [Person]
    def initialize(data, nationality, seed)
      @nationality = nationality
      @seed = seed
      convert(data)
    end

    # Converts a hash to instance variables
    #
    # @param [Hash] data the data used to create the instance variables
    def convert(data)
      data.each do |k, v|
        k = KEYS[k] if KEYS.include?(k)
        v = v.to_s if k.eql? 'zip'

        if NIDT.include?(k)
          instance_variable_set('@nidt', k)
          k = 'nidn'
          v = v.to_s
        end

        var_set(k, v)
      end
    end

    # Sets all instance variables
    #
    # @param [String] k the key used to create the instance variables
    # @param [String] v the value used to create the instance variables
    def var_set(k, v)
      varget = proc { instance_variable_get("@#{k}") }
      varset = proc { |y| instance_variable_set("@#{k}", y) }
      v.is_a?(Hash) ? convert(v) : instance_variable_set("@#{k}", v)
      self.class.send(:define_method, k, varget)
      self.class.send(:define_method, "#{k}=", varset)
    end
  end
end
